@@ -1,9 +1,9 @@
 from __future__ import division
+from collections import namedtuple
 import numpy as np
 from scipy.special import factorial
 from numdifftools.extrapolation import EPS, dea3
 from numdifftools.limits import _Limit
-from collections import namedtuple
 
 _INFO = namedtuple('info', ['error_estimate',
                             'degenerate',
@@ -79,17 +79,17 @@ def fd_weights_all(x, x0=0, n=1):
 # @jit(void(float64[:,:], float64[:], float64, int64))
 def _fd_weights_all(weights, x, x0, n):
     m = len(x)
-    c1, c4 = 1, x[0] - x0
+    c_1, c_4 = 1, x[0] - x0
     weights[0, 0] = 1
     for i in range(1, m):
         j = np.arange(0, min(i, n) + 1)
-        c2, c5, c4 = 1, c4, x[i] - x0
+        c_2, c_5, c_4 = 1, c_4, x[i] - x0
         for v in range(i):
-            c3 = x[i] - x[v]
-            c2, c6, c7 = c2 * c3, j * weights[v, j - 1], weights[v, j]
-            weights[v, j] = (c4 * c7 - c6) / c3
-        weights[i, j] = c1 * (c6 - c5 * c7) / c2
-        c1 = c2
+            c_3 = x[i] - x[v]
+            c_2, c_6, c_7 = c_2 * c_3, j * weights[v, j - 1], weights[v, j]
+            weights[v, j] = (c_4 * c_7 - c_6) / c_3
+        weights[i, j] = c_1 * (c_6 - c_5 * c_7) / c_2
+        c_1 = c_2
 
 
 def fd_weights(x, x0=0, n=1):
@@ -274,24 +274,24 @@ def _extrapolate(bs, rs, m):
     return extrap
 
 
-def _get_best_taylor_coefficients(bs, rs, m):
+def _get_best_taylor_coefficients(bs, rs, m, max_m1m2):
     extrap = _extrapolate(bs, rs, m)
     mvec = np.arange(m)
     if len(extrap) > 2:
         all_coefs, all_errors = dea3(extrap[:-2], extrap[1:-1], extrap[2:])
         steps = np.atleast_1d(rs[4:])[:, None] * mvec
+        # pylint: disable=protected-access
         coefs, info = _Limit._get_best_estimate(all_coefs, all_errors, steps, (m,))
         errors = info.error_estimate
     else:
-        errors = EPS / np.power(rs[2], mvec) * np.maximum(m1, m2)
+        errors = EPS / np.power(rs[2], mvec) * max_m1m2()
         coefs = extrap[-1]
     return coefs, errors
 
 
-def _check_fft(bnc, m, check_degenerate=True):
-    # If not degenerate, check for geometric progression in the FFT:
-    m1 = np.max(np.abs(bnc[:m // 2]))
-    m2 = np.max(np.abs(bnc[m // 2:]))
+def _check_fft(m1, m2, check_degenerate=True):
+    # If not degenerate, check for geometric progression in the FFT by comparing m1 and m2:
+
     # If there's an extreme mismatch, then we can consider the
     # geometric progression degenerate, whether one way or the other,
     # and just alternate directions instead of trying to target a
@@ -317,7 +317,7 @@ class Taylor(object):
     z0 : real or complex scalar at which to evaluate the derivatives
     n : scalar integer, default 1
         Number of taylor coefficents to compute. Maximum number is 100.
-    r : real scalar, default 0.0061
+    r : real scalar, default 0.0059
         Initial radius at which to evaluate. For well-behaved functions,
         the computation should be insensitive to the initial radius to within
         about four orders of magnitude.
@@ -359,8 +359,7 @@ class Taylor(object):
     magnitude at least. For most cases, the default configuration is likely to
     succeed.
 
-    Restrictions
-
+    Restrictions:
     The method uses the coefficients themselves to control the truncation
     error, so the error will not be properly bounded for functions like
     low-order polynomials whose Taylor series coefficients are nearly zero.
@@ -370,8 +369,8 @@ class Taylor(object):
 
     Examples
     --------
-
     Compute the first 6 taylor coefficients 1 / (1 - z) expanded round  z0 = 0:
+
     >>> import numdifftools.fornberg as ndf
     >>> import numpy as np
     >>> c, info = ndf.Taylor(lambda x: 1./(1-x), n=6, full_output=True)(z0=0)
@@ -379,10 +378,9 @@ class Taylor(object):
     True
     >>> np.all(info.error_estimate < 1e-9)
     True
-    >>> (info.function_count, info.iterations, info.failed) == (144, 18, False)
+    >>> (info.function_count, info.iterations, info.failed) == (136, 17, False)
     True
 
-
     References
     ----------
     [1] Fornberg, B. (1981).
@@ -391,11 +389,11 @@ class Taylor(object):
         7(4), 512-526. http://doi.org/10.1145/355972.355979
     """
 
-    def __init__(self, fun, n=1, r=0.0061, num_extrap=3, step_ratio=1.6, **kwds):
+    def __init__(self, fun, n=1, r=0.0059, num_extrap=3, step_ratio=1.6, **kwds):
         self.fun = fun
-        self.max_iter = kwds.get('max_iter', 30)
-        self.min_iter = kwds.get('min_iter', self.max_iter // 2)
-        self.full_output = kwds.get('full_output', False)
+        self.max_iter = kwds.pop('max_iter', 30)
+        self.min_iter = kwds.pop('min_iter', self.max_iter // 2)
+        self.full_output = kwds.pop('full_output', False)
         self.n = n
         self.r = r
         self.num_extrap = num_extrap
@@ -415,6 +413,17 @@ class Taylor(object):
         self._num_changes = 0
         return m, self._mvec
 
+    def _get_max_m1m2(self, bn, m):
+        m1, m2 = self._get_m1_m2(bn, m)
+        return np.maximum(m1, m2)
+
+    def _get_m1_m2(self, bn, m):
+        # If not degenerate, check for geometric progression in the FFT:
+        bnc = bn / self._crat
+        m1 = np.max(np.abs(bnc[:m // 2]))
+        m2 = np.max(np.abs(bnc[m // 2:]))
+        return m1, m2
+
     def _check_convergence(self, i, z0, r, m, bn):
         if self._direction_changes > 1 or self._degenerate:
             self._num_changes += 1
@@ -422,8 +431,11 @@ class Taylor(object):
                 return True, r
 
         if not self._degenerate:
-            self._degenerate, needs_smaller = _check_fft(bn / self._crat, m,
-                                                         check_degenerate=i > self.min_iter)
+            m1, m2 = self._get_m1_m2(bn, m)
+            # Note: only consider it degenerate if we've had a chance to steer
+            # the radius in the direction at least `min_iter` times:
+            check_degenerate = i > self.min_iter
+            self._degenerate, needs_smaller = _check_fft(m1, m2, check_degenerate)
             needs_smaller = needs_smaller or _poor_convergence(z0, r, self.fun, bn, self._mvec)
         if self._degenerate:
             needs_smaller = i % 2 == 0
@@ -467,16 +479,16 @@ class Taylor(object):
             if converged:
                 break
 
-        failed = not converged
-        coefs, errors = _get_best_taylor_coefficients(bs, rs, m)
+        coefs, errors = _get_best_taylor_coefficients(bs, rs, m, lambda: self._get_max_m1m2(bn, m))
         if self.full_output:
+            failed = not converged
             info = _INFO(errors, self._degenerate, final_radius=r,
                          function_count=i * m, iterations=i, failed=failed)
             return coefs, info
         return coefs
 
 
-def taylor(fun, z0=0, n=1, r=0.0061, num_extrap=3, step_ratio=1.6, **kwds):
+def taylor(fun, z0=0, n=1, r=0.0059, num_extrap=3, step_ratio=1.6, **kwds):
     """
     Return Taylor coefficients of complex analytic function using FFT
 
@@ -487,7 +499,7 @@ def taylor(fun, z0=0, n=1, r=0.0061, num_extrap=3, step_ratio=1.6, **kwds):
     z0 : real or complex scalar at which to evaluate the derivatives
     n : scalar integer, default 1
         Number of taylor coefficents to compute. Maximum number is 100.
-    r : real scalar, default 0.0061
+    r : real scalar, default 0.0059
         Initial radius at which to evaluate. For well-behaved functions,
         the computation should be insensitive to the initial radius to within
         about four orders of magnitude.
@@ -529,8 +541,7 @@ def taylor(fun, z0=0, n=1, r=0.0061, num_extrap=3, step_ratio=1.6, **kwds):
     magnitude at least. For most cases, the default configuration is likely to
     succeed.
 
-    Restrictions
-
+    Restrictions:
     The method uses the coefficients themselves to control the truncation
     error, so the error will not be properly bounded for functions like
     low-order polynomials whose Taylor series coefficients are nearly zero.
@@ -540,8 +551,8 @@ def taylor(fun, z0=0, n=1, r=0.0061, num_extrap=3, step_ratio=1.6, **kwds):
 
     Examples
     --------
-
     Compute the first 6 taylor coefficients 1 / (1 - z) expanded round  z0 = 0:
+
     >>> import numdifftools.fornberg as ndf
     >>> import numpy as np
     >>> c, info = ndf.taylor(lambda x: 1./(1-x), z0=0, n=6, full_output=True)
@@ -549,7 +560,7 @@ def taylor(fun, z0=0, n=1, r=0.0061, num_extrap=3, step_ratio=1.6, **kwds):
     True
     >>> np.all(info.error_estimate < 1e-9)
     True
-    >>> (info.function_count, info.iterations, info.failed) == (144, 18, False)
+    >>> (info.function_count, info.iterations, info.failed) == (136, 17, False)
     True
 
 
@@ -598,7 +609,7 @@ def derivative(fun, z0, n=1, **kwds):
     Returns
     -------
     der : ndarray
-       array of derivatives
+        array of derivatives
     status: Optional object into which output information is written. Fields:
         degenerate: True if the algorithm was unable to bound the error
         iterations: Number of iterations executed
@@ -607,6 +618,8 @@ def derivative(fun, z0, n=1, **kwds):
         failed: True if the maximum number of iterations was reached
         error_estimate: approximate bounds of the rounding error.
 
+    Notes
+    -----
     This module uses the method of Fornberg to compute the derivatives of a
     complex analytic function along with error bounds. The method uses a
     Fast Fourier Transform to invert function evaluations around a circle into
@@ -617,8 +630,7 @@ def derivative(fun, z0, n=1, **kwds):
     radius-to within a number of orders of magnitude at least. For most cases,
     the default configuration is likely to succeed.
 
-    Restrictions
-
+    Restrictions:
     The method uses the coefficients themselves to control the truncation
     error, so the error will not be properly bounded for functions like
     low-order polynomials whose Taylor series coefficients are nearly zero.
@@ -628,9 +640,9 @@ def derivative(fun, z0, n=1, **kwds):
 
     Examples
     --------
-
     To compute the first five derivatives of 1 / (1 - z) at z = 0:
     Compute the first 6 taylor derivatives of 1 / (1 - z) at z0 = 0:
+
     >>> import numdifftools.fornberg as ndf
     >>> import numpy as np
     >>> def fun(x):
@@ -640,7 +652,7 @@ def derivative(fun, z0, n=1, **kwds):
     True
     >>> np.all(info.error_estimate < 1e-9*c.real)
     True
-    >>> (info.function_count, info.iterations, info.failed) == (144, 18, False)
+    >>> (info.function_count, info.iterations, info.failed) == (136, 17, False)
     True
 
 
