@@ -9,10 +9,10 @@ Release date: 5/23/2008
 
 """
 from __future__ import absolute_import, division, print_function
-import numpy as np
 from collections import namedtuple
 from functools import partial
 import warnings
+import numpy as np
 from numdifftools.step_generators import MinStepGenerator
 from numdifftools.extrapolation import Richardson, dea3
 
@@ -28,8 +28,7 @@ class CStepGenerator(MinStepGenerator):
     Generates a sequence of steps
 
     where
-        steps = base_step * step_nom *
-                (exp(1j*dtheta) * step_ratio) ** (i + offset)
+        steps = base_step * step_nom * (exp(1j*dtheta) * step_ratio) ** (i + offset)
     for i = 0, 1, ..., num_steps-1
 
     Parameters
@@ -41,40 +40,38 @@ class CStepGenerator(MinStepGenerator):
     num_steps : scalar integer, optional,
         defines number of steps generated.
         If None the value is 2 * int(round(16.0/log(abs(step_ratio)))) + 1
-    step_nom :  default maximum(log(1+|x|), 1)
-        Nominal step where x is supplied at runtime through the __call__
-        method.
+    step_nom :  default maximum(log(exp(1)+|x|), 1)
+        Nominal step where x is supplied at runtime through the __call__ method.
     offset : real scalar, optional, default 0
         offset to the base step
-    use_exact_steps : boolean
-        If true make sure exact steps are generated
+    use_exact_steps : boolean, default True.
+        If true make sure exact steps are generated.
     scale : real scalar, default 1.2
         scale used in base step.
-    path : 'spiral' or 'radial'
-        Specifies the type of path to take the limit along.
-    dtheta: real scalar
-        If the path is spiral it will follow an exponential spiral into the
+    path : 'radial' or 'spiral'
+        Specifies the type of path to take the limit along. Default 'radial'.
+    dtheta: real scalar, default pi/8
+        If the path is 'spiral' it will follow an exponential spiral into the
         limit, with angular steps at dtheta radians.
 
     """
 
-    def __init__(self, base_step=None, step_ratio=4.0, num_steps=None,
-                 step_nom=None, offset=0, scale=1.2, use_exact_steps=True,
-                 path='radial', dtheta=np.pi / 8, **kwds):
-        self.path = path
-        self.dtheta = dtheta
+    def __init__(self, base_step=None, step_ratio=4.0, num_steps=None, step_nom=None,
+                 offset=0, scale=1.2, **options):
+        self.path = options.pop('path', 'radial')
+        self.dtheta = options.pop('dtheta', np.pi / 8)
         super(CStepGenerator,
               self).__init__(base_step=base_step, step_ratio=step_ratio,
-                             num_steps=num_steps, offset=offset, scale=scale,
-                             use_exact_steps=use_exact_steps, **kwds)
+                             num_steps=num_steps, step_nom=step_nom, offset=offset, scale=scale,
+                             **options)
         self._check_path()
 
     def _check_path(self):
-        _assert(self.path in ['spiral', 'radial'],
-                'Invalid Path: {}'.format(str(self.path)))
+        _assert(self.path in ['spiral', 'radial'], 'Invalid Path: {}'.format(str(self.path)))
 
     @property
     def step_ratio(self):
+        """Ratio between sequential steps generated."""
         dtheta = self.dtheta
         _step_ratio = float(self._step_ratio)  # radial path
         if dtheta != 0:
@@ -87,6 +84,7 @@ class CStepGenerator(MinStepGenerator):
 
     @property
     def dtheta(self):
+        """Angular steps in radians used for the exponential spiral path."""
         radial_path = self.path[0].lower() == 'r'
         return 0 if radial_path else self._dtheta
 
@@ -96,6 +94,7 @@ class CStepGenerator(MinStepGenerator):
 
     @property
     def num_steps(self):
+        """The number of steps generated"""
         if self._num_steps is None:
             return 2 * int(np.round(16.0 / np.log(np.abs(self.step_ratio)))) + 1
         return self._num_steps
@@ -111,20 +110,21 @@ class _Limit(object):
 
     info = namedtuple('info', ['error_estimate', 'final_step', 'index'])
 
-    def __init__(self, fun, step=None, method='above', order=4, full_output=False, **options):
-        self.fun = fun
-        self.method = method
-        self.order = order
-        self.full_output = full_output
+    def __init__(self, step=None, **options):
+
         self.step = step, options
 
-    def _parse_step_options(self, step):
+        self.richardson = Richardson(step_ratio=1.6, step=1, order=1, num_terms=2)
+
+    @staticmethod
+    def _parse_step_options(step):
         options = {}
         if isinstance(step, tuple) and isinstance(step[-1], dict):
             step, options = step
         return step, options
 
-    def _step_generator(self, step, options):
+    @staticmethod
+    def _step_generator(step, options):
         if hasattr(step, '__call__'):
             return step
         step_nom = None if step is None else 1
@@ -132,6 +132,7 @@ class _Limit(object):
 
     @property
     def step(self):
+        """The step spacing(s) used in the approximation"""
         return self._step
 
     @step.setter
@@ -156,10 +157,12 @@ class _Limit(object):
 
     @staticmethod
     def _add_error_to_outliers(der, trim_fact=10):
-        # discard any estimate that differs wildly from the
-        # median of all estimates. A factor of 10 to 1 in either
-        # direction is probably wild enough here. The actual
-        # trimming factor is defined as a parameter.
+        """
+        discard any estimate that differs wildly from the
+        median of all estimates. A factor of 10 to 1 in either
+        direction is probably wild enough here. The actual
+        trimming factor is defined as a parameter.
+        """
         try:
             median = np.nanmedian(der, axis=0)
             p75 = np.nanpercentile(der, 75, axis=0)
@@ -179,10 +182,10 @@ class _Limit(object):
     @staticmethod
     def _get_best_estimate(der, errors, steps, shape):
         errors += _Limit._add_error_to_outliers(der)
-        ix = _Limit._get_arg_min(errors)
-        final_step = steps.flat[ix].reshape(shape)
-        err = errors.flat[ix].reshape(shape)
-        return der.flat[ix].reshape(shape), _Limit.info(err, final_step, ix)
+        idx = _Limit._get_arg_min(errors)
+        final_step = steps.flat[idx].reshape(shape)
+        err = errors.flat[idx].reshape(shape)
+        return der.flat[idx].reshape(shape), _Limit.info(err, final_step, idx)
 
     @staticmethod
     def _wynn_extrapolate(der, steps):
@@ -190,8 +193,10 @@ class _Limit(object):
         return der, errors, steps[2:]
 
     def _extrapolate(self, results, steps, shape):
-        # if len(results)>2:
-        #     der1, errors1, steps = self._wynn_extrapolate(results, steps)
+        # if len(results) > 2:
+        #     der0, errors0, steps0 = self._wynn_extrapolate(results, steps)
+        #     if len(der0) > 0:
+        #         der2, errors2, steps2 = self._wynn_extrapolate(der0, steps0)
         # else:
         der1, errors1, steps = self.richardson(results, steps)
         if len(der1) > 2:
@@ -202,8 +207,9 @@ class _Limit(object):
     @staticmethod
     def _vstack(sequence, steps):
         original_shape = np.shape(sequence[0])
-        f_del = np.vstack([list(np.ravel(r)) for r in sequence])
-        h = np.vstack([list(np.ravel(np.ones(original_shape) * step)) for step in steps])
+        f_del = np.vstack([np.ravel(r) for r in sequence])
+        one = np.ones(original_shape)
+        h = np.vstack([np.ravel(one * step) for step in steps])
         _assert(f_del.size == h.size, 'fun did not return data of correct '
                 'size (it must be vectorized)')
         return f_del, h, original_shape
@@ -339,11 +345,18 @@ class Limit(_Limit):
 
     """
 
-    def _fun(self, z, dz, args, kwds):
-        return self.fun(z + dz, *args, **kwds)
+    def __init__(self, fun, step=None, method='above', order=4, full_output=False, **options):
+        super(Limit, self).__init__(step=step,  **options)
+        self.fun = fun
+        self.method = method
+        self.order = order
+        self.full_output = full_output
 
-    def _get_steps(self, xi):
-        return [step for step in self.step(xi)]
+    def _fun(self, z, d_z, args, kwds):
+        return self.fun(z + d_z, *args, **kwds)
+
+    def _get_steps(self, x_i):
+        return list(self.step(x_i))  # pylint: disable=not-callable
 
     def _set_richardson_rule(self, step_ratio, num_terms=2):
         self.richardson = Richardson(step_ratio=step_ratio, step=1, order=1,
@@ -351,51 +364,50 @@ class Limit(_Limit):
 
     def _lim(self, f, z):
         sign = dict(forward=1, above=1, backward=-1, below=-1)[self.method]
-        steps = [sign * step for step in self.step(z)]
-
+        steps = [sign * step for step in self.step(z)]  # pylint: disable=not-callable
+        # pylint: disable=no-member
         self._set_richardson_rule(self.step.step_ratio, self.order + 1)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            sequence = [f(z, h) for h in steps]
+        sequence = [f(z, h) for h in steps]
         results = self._vstack(sequence, steps)
         lim_fz, info = self._extrapolate(*results)
         return lim_fz, info
 
     def limit(self, x, *args, **kwds):
+        """Return lim f(z) as z-> x"""
         z = np.asarray(x)
         f = partial(self._fun, args=args, kwds=kwds)
-        fz, info = self._lim(f, z)
+        f_z, info = self._lim(f, z)
         if self.full_output:
-            return fz, info
-        return fz
+            return f_z, info
+        return f_z
 
-    def _call_lim(self, fz, z, f):
-        err = np.zeros_like(fz)
-        final_step = np.zeros_like(fz)
-        index = np.zeros_like(fz, dtype=int)
-        k = np.flatnonzero(np.isnan(fz))
+    def _call_lim(self, f_z, z, f):
+        err = np.zeros_like(f_z, dtype=float)
+        final_step = np.zeros_like(f_z)
+        index = np.zeros_like(f_z, dtype=int)
+        k = np.flatnonzero(np.isnan(f_z))
         if k.size > 0:
-            fz = np.where(np.isnan(fz), 0, fz)
             lim_fz, info1 = self._lim(f, z.flat[k])
-            np.put(fz, k, lim_fz)
+            zero = np.zeros(1, dtype=np.result_type(lim_fz))
+            f_z = np.where(np.isnan(f_z), zero, f_z)
+            np.put(f_z, k, lim_fz)
             if self.full_output:
+                final_step = np.where(np.isnan(f_z), zero, final_step)
                 np.put(final_step, k, info1.final_step)
                 np.put(index, k, info1.index)
                 np.put(err, k, info1.error_estimate)
-        return fz, self.info(err, final_step, index)
+        return f_z, self.info(err, final_step, index)
 
     def __call__(self, x, *args, **kwds):
         z = np.asarray(x)
         f = partial(self._fun, args=args, kwds=kwds)
-        with warnings.catch_warnings():
-            warnings.simplefilter("ignore")
-            fz = f(z, 0)
-
-        fz, info = self._call_lim(fz, z, f)
+        with np.errstate(divide='ignore', invalid='ignore'):
+            f_z = f(z, 0)
+            f_z, info = self._call_lim(f_z, z, f)
 
         if self.full_output:
-            return fz, info
-        return fz
+            return f_z, info
+        return f_z
 
 
 class Residue(Limit):
@@ -463,6 +475,7 @@ class Residue(Limit):
     --------
     A first order pole at z = 0
 
+    >>> import numpy as np
     >>> from numdifftools.limits import Residue
     >>> def f(z): return -1./(np.expm1(2*z))
     >>> res_f, info = Residue(f, full_output=True)(0)
@@ -493,8 +506,8 @@ class Residue(Limit):
         super(Residue, self).__init__(f, step=step, method=method, order=order,
                                       full_output=full_output, **options)
 
-    def _fun(self, z, dz, args, kwds):
-        return self.fun(z + dz, *args, **kwds) * (dz ** self.pole_order)
+    def _fun(self, z, d_z, args, kwds):
+        return self.fun(z + d_z, *args, **kwds) * (d_z ** self.pole_order)
 
     def __call__(self, x, *args, **kwds):
         return self.limit(x, *args, **kwds)
