@@ -57,13 +57,13 @@ from __future__ import division
 from collections import namedtuple
 
 from scipy import special
-
 import numpy as np
 try:
     import algopy
     from algopy import UTPM
 except ImportError:
-    algopy = None
+    UTPM = algopy = None
+
 
 EPS = np.MachAr().eps
 
@@ -72,19 +72,19 @@ _cmn_doc = """
 
     Parameters
     ----------
-    fun : function
+    fun: function
         function of one array fun(x, `*args`, `**kwds`)%(extra_parameter)s
-    method : string, optional {'forward', 'reverse'}
+    method: string, optional {'forward', 'reverse'}
         defines method used in the approximation
 
     Methods
     -------
-    __call__ : callable with the following parameters:
-        x : array_like
+    __call__: callable with the following parameters:
+        x: array_like
            value at which function derivative is evaluated
-        args : tuple
+        args: tuple
             Arguments for function `fun`.
-        kwds : dict
+        kwds: dict
             Keyword arguments for function `fun`.
     %(returns)s
     Notes
@@ -169,12 +169,12 @@ class Derivative(_Derivative):
     __doc__ = _cmn_doc % dict(
         derivative='n-th derivative',
         extra_parameter="""
-    n : int, optional
+    n: int, optional
         Order of the derivative.""",
         extra_note="", returns="""
     Returns
     -------
-    der : ndarray
+    der: ndarray
        array of derivatives
     """, example="""
     Examples
@@ -236,11 +236,12 @@ class Gradient(_Derivative):
         extra_note="", returns="""
     Returns
     -------
-    grad : array
+    grad: array
         gradient
     """, example="""
     Examples
     --------
+    >>> import numpy as np
     >>> import numdifftools.nd_algopy as nda
     >>> fun = lambda x: np.sum(x**2)
     >>> df = nda.Gradient(fun, method='reverse')
@@ -278,9 +279,9 @@ class Gradient(_Derivative):
     def _forward(self, x, *args, **kwds):
         # forward mode without building the computational graph
 
-        tmp = algopy.UTPM.init_jacobian(x)
+        tmp = UTPM.init_jacobian(x)
         y = self.fun(tmp, *args, **kwds)
-        return algopy.UTPM.extract_jacobian(y)
+        return UTPM.extract_jacobian(y)
 
     def _reverse(self, x, *args, **kwds):
 
@@ -296,11 +297,12 @@ class Jacobian(Gradient):
         extra_note="", returns="""
     Returns
     -------
-    jacob : array
+    jacob: array
         Jacobian
     """, example="""
     Examples
     --------
+    >>> import numpy as np
     >>> import numdifftools.nd_algopy as nda
 
     #(nonlinear least squares)
@@ -382,6 +384,7 @@ class Hessian(_Derivative):
     """, extra_note='', example="""
     Examples
     --------
+    >>> import numpy as np
     >>> import numdifftools.nd_algopy as nda
 
     # Rosenbrock function, minimized at [1,1]
@@ -423,9 +426,9 @@ class Hessian(_Derivative):
 
     def _forward(self, x, *args, **kwds):
         x = np.atleast_1d(x)
-        tmp = algopy.UTPM.init_hessian(x)
+        tmp = UTPM.init_hessian(x)
         y = self.fun(tmp, *args, **kwds)
-        return algopy.UTPM.extract_hessian(len(x), y)
+        return UTPM.extract_hessian(len(x), y)
 
     def _reverse(self, x, *args, **kwds):
         x = np.atleast_1d(np.asarray(x, dtype=float))
@@ -446,6 +449,7 @@ class Hessdiag(Hessian):
     """, extra_note='', example="""
     Examples
     --------
+    >>> import numpy as np
     >>> import numdifftools.nd_algopy as nda
 
     # Rosenbrock function, minimized at [1,1]
@@ -523,14 +527,12 @@ def directionaldiff(f, x0, vec, **options):
     compute the directional derivative in the direction [1 2]
 
     >>> import numpy as np
-    >>> import numdifftools as nd
+    >>> import numdifftools.nd_algopy as nda
     >>> vec = np.r_[1, 2]
     >>> rosen = lambda x: (1-x[0])**2 + 105*(x[1]-x[0]**2)**2
-    >>> dd, info = nd.directionaldiff(rosen, [1, 1], vec, full_output=True)
+    >>> dd = nda.directionaldiff(rosen, [1, 1], vec)
     >>> np.allclose(dd, 0)
     True
-    >>> np.abs(info.error_estimate)<1e-14
-    True
 
     See also
     --------
@@ -546,6 +548,53 @@ def directionaldiff(f, x0, vec, **options):
     return Derivative(lambda t: f(x0 + t * vec), **options)(0)
 
 
+class Taylor(object):
+
+    """
+    Return Taylor coefficients of function using algorithmic differentiation
+
+    Parameters
+    ----------
+    fun: callable
+        function to differentiate
+    z0: real or complex array
+        at which to evaluate the derivatives
+    n: scalar integer, default 1
+        Number of taylor coefficents to compute.
+
+    Returns
+    -------
+    coefs: ndarray
+       array of taylor coefficents
+
+    Examples
+    --------
+    Compute the first 6 taylor coefficients 1 + 2*z + 3*z**2 expanded round  z0 = 0:
+
+    >>> import numpy as np
+    >>> import numdifftools.nd_algopy as nda
+    >>> c = nda.Taylor(lambda x: 1+2*x+3*x**2, n=6)(z0=0)
+    >>> np.allclose(c, [1, 2, 3, 0, 0, 0])
+    True
+
+    """
+
+    def __init__(self, fun, n=1):
+        self.fun = fun
+        self.n = n
+
+    def __call__(self, z0=0):
+        z = np.atleast_1d(z0).ravel()
+        x = UTPM(np.zeros((self.n, 1, z.size), dtype=z.dtype))
+
+        x.data[0, 0, :] = z
+        x.data[1, 0, :] = 1
+
+        y = self.fun(x)
+        coefs = np.squeeze(y.data)
+        return coefs
+
+
 if __name__ == '__main__':
     from numdifftools.testing import test_docstrings
     test_docstrings()
