@@ -2,7 +2,7 @@
 from __future__ import print_function
 
 from hypothesis import given, example, note, settings, strategies as st
-from numpy.testing import assert_allclose
+from numpy.testing import assert_allclose  # @UnresolvedImport
 import pytest
 
 import numdifftools.core as nd
@@ -146,8 +146,7 @@ class TestDerivative(object):
     @staticmethod
     def test_high_order_derivative_cos():
         true_vals = (0.0, -1.0, 0.0, 1.0, 0.0, -1.0, 0.0)
-        methods = ['complex', 'multicomplex', 'central',
-                   'forward', 'backward']
+        methods = ['complex', 'multicomplex', 'central', 'forward', 'backward']
         for method in methods:
             n_max = dict(multicomplex=2, central=6).get(method, 5)
             for n in range(0, n_max + 1):
@@ -171,6 +170,8 @@ class TestDerivative(object):
     @settings(deadline=500.0)
     @given(st.floats(min_value=0, max_value=10))
     @example(7.6564547238847105)
+    @example(8.9428143931508)
+    @example(2.2204460492503134e-14)
     def test_derivative_of_cos_x(x):
         note('x = {}'.format(x))
         msg = 'order = {}, error = {}, err_est = {}'
@@ -198,9 +199,8 @@ class TestDerivative(object):
 
     @staticmethod
     def test_default_scale():
-        for method, scale in zip(['complex', 'central', 'forward', 'backward',
-                                  'multicomplex'],
-                                 [1.35, 2.5, 2.5, 2.5, 1.35]):
+        for method, scale in zip(['complex', 'central', 'forward', 'backward', 'multicomplex'],
+                                 [1.06, 2.5, 2.5, 2.5, 1.06]):
             assert_allclose(scale, default_scale(method, n=1))
 
     @staticmethod
@@ -211,14 +211,13 @@ class TestDerivative(object):
             return b * a * x * x * x
 
         methods = ['forward', 'backward', 'central', 'complex', 'multicomplex']
-        dfuns = [nd.Gradient, nd.Derivative, nd.Jacobian, nd.Hessdiag,
-                 nd.Hessian]
+        dfuns = [nd.Gradient, nd.Derivative, nd.Jacobian, nd.Hessdiag, nd.Hessian]
         truths = {nd.Hessdiag: 12, nd.Hessian: 12}
         for dfun in dfuns:
             for method in methods:
                 df = dfun(func, method=method)
                 val = df(0.0, 1.0, b=2)
-                assert_allclose(val, 0, atol=1e-14)
+                assert_allclose(val, 0, atol=1e-13)
 
                 val = df(1.0, 1.0, b=2)
                 truth = truths.get(dfun, 6)
@@ -231,14 +230,13 @@ class TestDerivative(object):
             return b * a * x * x * x
 
         methods = ['forward', 'backward', 'central', 'complex', 'multicomplex']
-        dfuns = [nd.Gradient, nd.Derivative, nd.Jacobian, nd.Hessdiag,
-                 nd.Hessian]
+        dfuns = [nd.Gradient, nd.Derivative, nd.Jacobian, nd.Hessdiag, nd.Hessian]
         step_options = dict(num_extrap=5)
         for dfun in dfuns:
             for method in methods:
                 df = dfun(func, method=method, **step_options)
                 val = df(0.0, 1.0, b=2)
-                assert_allclose(val, 0, atol=1e-14)
+                assert_allclose(val, 0, atol=1e-13)
 
     @staticmethod
     def test_derivative_cube():
@@ -297,15 +295,18 @@ class TestJacobian(object):
     def test_scalar_to_vector(val):
 
         def fun(x):
-            return np.array([x, x ** 2, x ** 3])
+            return np.array([x, x ** 2, x ** 3]).ravel()
 
-        truth = np.array([[1., 2 * val, 3 * val ** 2]])
-        for method in ['multicomplex', 'complex', 'central', 'forward',
-                       'backward']:
+        truth = np.array([[1.], [2 * val], [3 * val ** 2]])
+        for method in ['multicomplex', 'complex', 'central', 'forward', 'backward']:
             j0, info = nd.Jacobian(fun, method=method, full_output=True)(val)
+            if method != "multicomplex":
+                j00 = nds.Jacobian(fun, method=method)(val)
+                error = np.abs(j00 - truth)
+                note('statsmodel: method={}, error={}'.format(method, error))
+                assert_allclose(j00, truth, rtol=1e-3, atol=1e-6)
             error = np.abs(j0 - truth)
-            note('method={}, error={}, error_est={}'.format(method, error,
-                                                            info.error_estimate))
+            note('method={}, error={}, error_est={}'.format(method, error, info.error_estimate))
             assert_allclose(j0, truth, rtol=1e-3, atol=1e-6)
 
     @staticmethod
@@ -327,7 +328,7 @@ class TestJacobian(object):
         def fun(c):
             return (c[0] + c[1] * np.exp(c[2] * xdata) - ydata) ** 2
 
-        _j_0 = nds.approx_fprime([1, 2, 0.75], fun)
+        # _j_0 = nds.approx_fprime([1, 2, 0.75], fun)
 
         for method in ['complex', 'central', 'forward', 'backward']:
             for order in [2, 4]:
@@ -386,8 +387,8 @@ class TestJacobian(object):
                                 [75., 108., 147., 192.]]])
         v0 = nds.approx_fprime(x, fun)
         val = jaca(x)
-        assert_allclose(v0, tval)
         assert_allclose(val, tval)
+        assert_allclose(v0, tval)
 
     @staticmethod
     def test_issue_25():
@@ -439,14 +440,33 @@ class TestJacobian(object):
     @staticmethod
     @pytest.mark.slow
     def test_issue_27b():
+        """Test for memory-error"""
         n = 1000
         x = np.ones(n)
         assert_allclose(nd.Jacobian(lambda x: x ** 2, method='complex')(x),
                         2 * np.diag(np.ones(n)))
 
+    @staticmethod
+    def test_jacobian_fulloutput():
+        """test """
+        res, info = nd.Jacobian(lambda x, y: x + y, full_output=True)(1, 3)
+        assert_allclose(res, 1)
+        assert info.error_estimate < 1e-13
+        assert info.final_step == 0.015625
+        assert info.index == 5
+        assert info.f_value == 4
+
 
 class TestGradient(object):
 
+    @staticmethod
+    def test_issue_39():
+        """
+        Test that checks float/Bicomplex works
+        """
+        fun = nd.Gradient(lambda x: 1.0/(np.exp(x[0]) + np.cos(x[1]) + 10), method="multicomplex")
+        assert_allclose(fun([1.0, 2.0]), [-0.017961123762187736, 0.0060082083648822])
+
     @staticmethod
     def test_directional_diff():
         v = np.r_[1, -1]
@@ -457,6 +477,20 @@ class TestGradient(object):
         dd, _info = nd.directionaldiff(rosen, x0, v, full_output=True)
         assert_allclose(dd, 743.87633380824832)
 
+    @staticmethod
+    def test_gradient_fulloutput():
+        """Fix issue#52:
+
+        Gradient tries to apply squeeze to the output tuple containing both the result
+        and the full_output object.
+        """
+        res, info = nd.Gradient(lambda x, y: x + y, full_output=True)(1, 3)
+        assert_allclose(res, 1)
+        assert info.error_estimate < 1e-13
+        assert info.final_step == 0.015625
+        assert info.index == 5
+        assert info.f_value == 4
+
     @staticmethod
     def test_gradient():
 
@@ -465,13 +499,11 @@ class TestGradient(object):
 
         dtrue = [2., 4., 6.]
 
-        for method in ['multicomplex', 'complex', 'central', 'backward',
-                       'forward']:
+        for method in ['multicomplex', 'complex', 'central', 'backward', 'forward']:
             for order in [2, 4]:
                 dfun = nd.Gradient(fun, method=method, order=order)
                 d = dfun([1, 2, 3])
                 assert_allclose(d, dtrue)
-        # assert False
 
 
 class TestHessdiag(object):
@@ -497,6 +529,7 @@ class TestHessdiag(object):
             h_val, _info = h_fun([1, 2, 3])
 
             assert_allclose(h_val, htrue)
+            assert _info.f_value == 32  # fun([1, 2, 3]) == 1+4+27
 
     @settings(deadline=500.0)
     @given(st.tuples(st.floats(-100, 100), st.floats(-100, 100), st.floats(-100, 100)))
@@ -504,10 +537,10 @@ class TestHessdiag(object):
     @example([1, 2, 3])
     @example((1.2009289063e-314, 1.2009289063e-314, 1.2009289063e-314))
     @example((88.01663712016305, 88.01663712016305, 88.01663712016305))
+    @example((0.0, 19.945812226807096, 54.11322414875562))
     def test_fixed_step(self, vals):
         htrue = self._hfun(vals)
-        methods = ['central2', 'central', 'multicomplex', 'complex', 'forward',
-                   'backward']
+        methods = ['central2', 'central', 'multicomplex', 'complex', 'forward', 'backward']
         for order in range(2, 7, 2):
             steps = nd.MinStepGenerator(num_steps=order + 1,
                                         use_exact_steps=True,
@@ -520,7 +553,7 @@ class TestHessdiag(object):
                 _error = np.abs(h_val - htrue)
                 note('error = {}, error_est = {}'.format(_error,
                                                          _info.error_estimate))
-                assert_allclose(h_val, htrue, atol=100 * max(_info.error_estimate))
+                assert_allclose(h_val, htrue, rtol=1e-5, atol=100 * max(_info.error_estimate))
 
     def test_default_step(self):
         htrue = np.array([0., 2., 18.])
@@ -531,7 +564,6 @@ class TestHessdiag(object):
                 h_fun = nd.Hessdiag(self._fun, method=method, order=order,
                                     full_output=True)
                 h_val, _info = h_fun([1, 2, 3])
-                # _error = h_val - htrue
                 tol = min(1e-8, _info.error_estimate.max())
                 assert_allclose(h_val, htrue, atol=tol)
 
@@ -568,7 +600,7 @@ class TestHessian(object):
 
                     assert_allclose(val, [[2j, 0j, 0j],
                                           [0j, 2j, 0j],
-                                          [0j, 0j, 2j]], atol=1e-13)
+                                          [0j, 0j, 2j]], atol=1e-11)
 
     @staticmethod
     def test_hessian_cos_x_y_at_0_0():
@@ -588,3 +620,4 @@ class TestHessian(object):
                 h_val, _info = h_fun([0, 0])
                 # print(method, (h_val-np.array(htrue)))
                 assert_allclose(h_val, htrue)
+                assert _info.f_value == 1
